@@ -1,7 +1,7 @@
 package RT::Extension::LDAPImport;
 
 # XXX TODO: For historical reasons, this should become 0.33 after 0.32_xx dev releases are done.
-our $VERSION = '0.32_03';
+our $VERSION = '0.32_05';
 
 use warnings;
 use strict;
@@ -18,27 +18,243 @@ use Data::Dumper;
 
 RT::Extension::LDAPImport - Import Users from an LDAP store
 
-
 =head1 SYNOPSIS
 
-    use RT::Extension::LDAPImport;
+In C<RT_SiteConfig.pm>:
+
+    Set($LDAPHost,'my.ldap.host')
+    Set($LDAPUser,'me');
+    Set($LDAPPassword,'mypass');
+    Set($LDAPFilter, '(&(cn = users))');
+    Set($LDAPMapping, {Name         => 'uid', # required
+                       EmailAddress => 'mail',
+                       RealName     => 'cn',
+                       WorkPhone    => 'telephoneNumber',
+                       Organization => 'departmentName'});
+    
+    # Add to any existing plugins
+    Set(@Plugins, qw(RT::Extension::LDAPImport));
+    
+    # If you want to sync Groups RT <-> LDAP
+    
+    Set($LDAPGroupBase, 'ou=Groups,o=Our Place');
+    Set($LDAPGroupFilter, '(&(cn = Groups))');
+    Set($LDAPGroupMapping, {Name               => 'cn',
+                            Member_Attr        => 'member',
+                            Member_Attr_Value  => 'dn' });
+
+Running the import:
+
+    # Run a test import
+    /opt/rt4/local/plugins/RT-Extension-LDAPImport/bin/rtldapimport \
+    --debug > ldapimport.debug 2>&1
+    
+    # Run for real, possibly put in cron
+    /opt/rt4/local/plugins/RT-Extension-LDAPImport/bin/rtldapimport \
+    --import
+
+=head1 CONFIGURATION
+
+All of the configuration for the importer goes
+your C<RT_SiteConfig.pm> file. Some of these values pass through
+to L<Net::LDAP> so you can check there for valid values and more
+advanced options.
+
+=over
+
+=item C<< Set($LDAPHost,'our.ldap.host'); >>
+
+Hostname or ldap(s):// uri:
+
+=item C<< Set($LDAPUser, 'uid=foo,ou=users,dc=example,dc=com'); >>
+
+Your LDAP username or DN. If unset, we'll attempt an anonymous bind.
+
+=item C<< Set($LDAPPassword, 'ldap pass'); >>
+
+Your LDAP password.
+
+=item C<< Set($LDAPBase, 'ou=People,o=Our Place'); >>
+
+Base object to search from.
+
+=item C<< Set($LDAPFilter, '(&(cn = users))'); >>
+
+The LDAP search filter to apply (in this case, find all the users).
+
+=item C<< Set($LDAPMapping... >>
+
+    Set($LDAPMapping, {Name         => 'uid',
+                       EmailAddress => 'mail',
+                       RealName     => 'cn',
+                       WorkPhone    => 'telephoneNumber',
+                       Organization => 'departmentName'});
+
+This provides the mapping of attributes in RT to attribute in LDAP.
+Only Name is required for RT.
+
+The LDAP attributes can also be an arrayref of LDAP fields
+
+    WorkPhone => [qw/CompanyPhone Extension/]
+
+which will be concatenated together with a space.
+
+The LDAP attribute can also be a subroutine reference
+that returns either an arrayref or a list of attributes.
+
+The keys in the mapping (i.e. the RT fields, the left hand side) may be a user
+custom field name prefixed with C<UserCF.>, for example C<< 'UserCF.Employee
+Number' => 'employeeId' >>.  Note that this only B<adds> values at the moment,
+which on single value CFs will remove any old value first.  Multiple value CFs
+may behave not quite how you expect.  If the attribute no longer exists on a
+user in LDAP, it will be cleared on the RT side as well.
+
+You may also prefix any RT custom field name with C<CF.> inside your mapping to
+add available values to a Select custom field.  This effectively takes user
+attributes in LDAP and adds the values as selectable options in a CF.  It does
+B<not> set a CF value on any RT object (User, Ticket, Queue, etc).  You might
+use this to populate a ticket Location CF with all the locations of your users
+so that tickets can be associated with the locations in use.
+
+=item C<< Set($LDAPCreatePrivileged, 1); >>
+
+By default users are created as Unprivileged, but you can change this by
+setting C<$LDAPCreatePrivileged> to 1.
+
+=item C<< Set($LDAPGroupName,'My Imported Users'); >>
+
+The RT Group new and updated users belong to. By default, all users
+added or updated by the importer will belong to the 'Imported from LDAP'
+group.
+
+=item C<< Set($LDAPSkipAutogeneratedGroup, 1); >>
+
+Set this to true to prevent users from being automatically
+added to the group configured by C<$LDAPGroupName>.
+
+=item C<< Set($LDAPUpdateUsers, 1); >>
+
+By default, existing users are skipped.  If you
+turn on LDAPUpdateUsers, we will clobber existing
+data with data from LDAP.
+
+=item C<< Set($LDAPUpdateOnly, 1); >>
+
+By default, we create users who don't exist in RT but do
+match your LDAP filter and obey C<$LDAPUpdateUsers> for existing
+users.  This setting updates existing users, overriding
+C<$LDAPUpdateUsers>, but won't create new
+users who are found in LDAP but not in RT.
+
+=item C<< Set($LDAPGroupBase, 'ou=Groups,o=Our Place'); >>
+
+Where to search for groups to import.
+
+=item C<< Set($LDAPGroupFilter, '(&(cn = Groups))'); >>
+
+The search filter to apply.
+
+=item C<< Set($LDAPGroupMapping... >>
+
+    Set($LDAPGroupMapping, {Name               => 'cn',
+                            Member_Attr        => 'member',
+                            Member_Attr_Value  => 'dn' });
+
+A mapping of RT attributes to LDAP attributes to identify group members.
+Name will become the name of the group in RT, in this case pulling
+from the cn attribute on the LDAP group record returned.
+
+C<Member_Attr> is the field in the LDAP group record the importer should
+look at for group members. These values (there may be multiple members)
+will then be compared to the RT user name, which came from the LDAP
+user record.
+
+C<Member_Attr_Value>, which defaults to 'dn', specifies where on the LDAP
+user record the importer should look to compare the member value.
+A match between the member field on the group record and this
+identifier (dn or other LDAP field) on a user record means the
+user will be added to that group in RT.
+
+You can provide a C<Description> key which will be added as the group
+description in RT. The default description is 'Imported from LDAP'.
+
+=item C<< Set($LDAPSizeLimit, 1000); >>
+
+You can set this value if your LDAP server has result size limits.
+
+=back
+
+=head1 Mapping Groups Between RT and LDAP
+
+If you are using the importer, you likely want to manage access via
+LDAP by putting people in groups like 'DBAs' and 'IT Support', but
+also have groups for other non-RT related things. In this case, you
+won't want to create all of your LDAP groups in RT. To limit the groups
+that get mirrored, construct your C<$LDAPGroupFilter> as an OR (|) with
+all of the RT groups you want to mirror from LDAP. For example:
+
+    Set($LDAPGroupBase, 'OU=Groups,OU=Company,DC=COM');
+    Set($LDAPGroupFilter, '(|(CN=DBAs)(CN=IT Support))');
+
+The importer will then import only the groups that match. In this case,
+import means:
+
+=over
+
+=item * Verifying the group is in AD;
+
+=item * Creating the group in RT if it doesn't exist;
+
+=item * Populating the group with the members identified in AD;
+
+=back
+
+The import script will also issue a warning if a user isn't found in RT,
+but this should only happen when testing. When running with --import on,
+users are created before groups are processed, so all users (group
+members) should exist unless there are inconsistencies in your LDAP configuration.
+
+=head1 Running the Import
+
+Executing C<rtldapimport> will run a test that connects to your LDAP server
+and prints out a list of the users found. To see more about these users,
+and to see more general debug information, include the C<--debug> flag.
+
+That debug information is also sent to the RT log with the debug level.
+Errors are logged to the screen and to the RT log.
+
+Executing C<rtldapimport> with the C<--import> flag will cause it to import
+users into your RT database. It is recommended that you make a database
+backup before doing this. If your filters aren't set properly this could
+create a lot of users or groups in your RT instance.
+
+=head1 RT Versions
+
+The importer works with RT 3.8 and newer including RT 4.
+
+It may work with RT 3.6.
+
+=head1 LDAP Filters
+
+The L<ldapsearch|http://www.openldap.org/software/man.cgi?query=ldapsearch&manpath=OpenLDAP+2.0-Release>
+utility in openldap can be very helpful while refining your filters.
 
 =head1 METHODS
 
 =head2 connect_ldap
 
-Relies on the config variables $RT::LDAPHost,
-$RT::LDAPUser and $RT::LDAPPassword being set
+Relies on the config variables C<$RT::LDAPHost>,
+C<$RT::LDAPUser> and C<$RT::LDAPPassword> being set
 in your RT Config files.
 
- Set(LDAPHost,'my.ldap.host')
- Set(LDAPUSER,'me');
- Set(LDAPPassword,'mypass');
+ Set($LDAPHost,'my.ldap.host')
+ Set($LDAPUSER,'me');
+ Set($LDAPPassword,'mypass');
 
 LDAPUser and LDAPPassword can be blank,
 which will cause an anonymous bind.
 
-LDAPHost can be a hostname or an ldap:// ldaps:// uri
+LDAPHost can be a hostname or an ldap:// ldaps:// uri.
 
 =cut
 
@@ -73,7 +289,7 @@ sub connect_ldap {
 
 =head2 run_user_search
 
-Set up the appropriate arguments for a listing of users
+Set up the appropriate arguments for a listing of users.
 
 =cut
 
@@ -88,9 +304,9 @@ sub run_user_search {
 
 =head2 _run_search
 
-Executes a search using the provided base and filter
+Executes a search using the provided base and filter.
 
-Will connect to LDAP server using connect_ldap
+Will connect to LDAP server using C<connect_ldap>.
 
 Returns an array of L<Net::LDAP::Entry> objects, possibly consolidated from
 multiple LDAP pages.
@@ -163,23 +379,23 @@ sub _run_search {
 =head2 import_users import => 1|0
 
 Takes the results of the search from run_search
-and maps attributes from LDAP into RT::User attributes
-using $RT::LDAPMapping.
+and maps attributes from LDAP into C<RT::User> attributes
+using C<$RT::LDAPMapping>.
 Creates RT users if they don't already exist.
 
 With no arguments, only prints debugging information.
-Pass import => 1 to actually change data.
+Pass C<--import> to actually change data.
 
-RT::LDAPMapping should be set in your RT_SiteConfig
-file and looks like this.
+C<$RT::LDAPMapping>> should be set in your C<RT_SiteConfig.pm>
+file and look like this.
 
  Set($LDAPMapping, { RTUserField => LDAPField, RTUserField => LDAPField });
 
-RTUserField is the name of a field on an RT::User object
+RTUserField is the name of a field on an C<RT::User> object
 LDAPField can be a simple scalar and that attribute
-will be looked up in LDAP.  
+will be looked up in LDAP.
 
-It can also be an arrayref, in which case each of the 
+It can also be an arrayref, in which case each of the
 elements will be evaluated in turn.  Scalars will be
 looked up in LDAP and concatenated together with a single
 space.
@@ -191,7 +407,7 @@ If it is an arrayref, the values will be concatenated
 together with a single space.
 
 By default users are created as Unprivileged, but you can change this by
-setting $LDAPCreatePrivileged to 1.
+setting C<$LDAPCreatePrivileged> to 1.
 
 =cut
 
@@ -231,7 +447,7 @@ sub import_users {
 
 =head2 _import_user
 
-The user has run us with --import, so bring data in
+The user has run us with --import, so bring data in.
 
 =cut
 
@@ -247,6 +463,7 @@ sub _import_user {
 
     $self->add_user_to_group( %args );
     $self->add_custom_field_value( %args );
+    $self->update_object_custom_field_values( %args, object => $args{user} );
 
     return 1;
 }
@@ -268,6 +485,8 @@ sub _cache_user {
     my %args = (@_);
     my $user = $args{user} || $self->_build_user_object( ldap_entry => $args{ldap_entry} );
 
+    $self->_users({}) if not defined $self->_users;
+
     my $group_map       = $RT::LDAPGroupMapping           || {};
     my $member_attr_val = $group_map->{Member_Attr_Value} || 'dn';
     my $membership_key  = lc $member_attr_val eq 'dn'
@@ -308,7 +527,7 @@ sub _show_user_info {
 
 =head2 _check_ldap_mapping
 
-Returns true is there is an LDAPMapping configured,
+Returns true is there is an C<LDAPMapping> configured,
 returns false, logs an error and disconnects from
 ldap if there is no mapping.
 
@@ -331,15 +550,16 @@ sub _check_ldap_mapping {
 
 =head2 _build_user_object
 
-Utility method which wraps _build_object to provide sane defaults for building
-users.  It also tries to ensure a Name exists in the returned object.
+Utility method which wraps C<_build_object> to provide sane
+defaults for building users.  It also tries to ensure a Name
+exists in the returned object.
 
 =cut
 
 sub _build_user_object {
     my $self = shift;
     my $user = $self->_build_object(
-        skip    => qr/(?i)^CF\./,
+        skip    => qr/(?i)^(?:User)?CF\./,
         mapping => $RT::LDAPMapping,
         @_
     );
@@ -350,7 +570,8 @@ sub _build_user_object {
 =head2 _build_object
 
 Builds up data from LDAP for importing
-Returns a hash of user or group data ready for RT::User::Create or RT::Group::Create
+Returns a hash of user or group data ready for
+C<RT::User::Create> or C<RT::Group::Create>.
 
 =cut
 
@@ -384,8 +605,8 @@ sub _build_object {
 
 =head3 _parse_ldap_mapping
 
-Internal helper function for import_user
-If we're passed an arrayref, it will recurse 
+Internal helper function for C<import_user>.
+If we're passed an arrayref, it will recurse
 over each of the elements in case one of them is
 another arrayref or subroutine.
 
@@ -418,16 +639,16 @@ sub _parse_ldap_mapping {
 
 =head2 create_rt_user
 
-Takes a hashref of args to pass to RT::User::Create
+Takes a hashref of args to pass to C<RT::User::Create>
 Will try loading the user and will only create a new
-user if it can't find an existing user with the Name
-or EmailAddress arg passed in.
+user if it can't find an existing user with the C<Name>
+or C<EmailAddress> arg passed in.
 
-If the $LDAPUpdateUsers variable is true, data in RT
+If the C<$LDAPUpdateUsers> variable is true, data in RT
 will be clobbered with data in LDAP.  Otherwise we
 will skip to the next user.
 
-If $LDAPUpdateOnly is true, we will not create new users
+If C<$LDAPUpdateOnly> is true, we will not create new users
 but we will update existing ones.
 
 =cut
@@ -498,9 +719,9 @@ sub _load_rt_user {
 
 =head2 add_user_to_group
 
-Adds new users to the group specified in the $LDAPGroupName
-variable (defaults to 'Imported from LDAP')
-You can avoid this if you set $LDAPSkipAutogeneratedGroup
+Adds new users to the group specified in the C<$LDAPGroupName>
+variable (defaults to 'Imported from LDAP').
+You can avoid this if you set C<$LDAPSkipAutogeneratedGroup>.
 
 =cut
 
@@ -536,8 +757,8 @@ sub add_user_to_group {
 
 =head2 setup_group
 
-Pulls the $LDAPGroupName object out of the DB or
-creates it if we ened to do so.
+Pulls the C<$LDAPGroupName> object out of the DB or
+creates it if we need to do so.
 
 =cut
 
@@ -563,7 +784,7 @@ Adds values to a Select (one|many) Custom Field.
 The Custom Field should already exist, otherwise
 this will throw an error and not import any data.
 
-This could probably use some caching
+This could probably use some caching.
 
 =cut
 
@@ -623,18 +844,64 @@ sub add_custom_field_value {
 
 }
 
+=head3 update_object_custom_field_values
+
+Adds CF values to an object (currently only users).  The Custom Field should
+already exist, otherwise this will throw an error and not import any data.
+
+Note that this code only B<adds> values at the moment, which on single value
+CFs will remove any old value first.  Multiple value CFs may behave not quite
+how you expect.
+
+=cut
+
+sub update_object_custom_field_values {
+    my $self = shift;
+    my %args = @_;
+    my $obj  = $args{object};
+
+    foreach my $rtfield ( keys %{$RT::LDAPMapping} ) {
+        # XXX TODO: accept GroupCF when we call this from group_import too
+        next unless $rtfield =~ /^UserCF\.(.+)$/i;
+        my $cf_name = $1;
+        my $ldap_attribute = $RT::LDAPMapping->{$rtfield};
+
+        my @attributes = $self->_parse_ldap_mapping($ldap_attribute);
+        unless (@attributes) {
+            $self->_error("Invalid LDAP mapping for $rtfield ".Dumper($ldap_attribute));
+            next;
+        }
+        my $value = join ' ',
+                    grep { defined and length }
+                     map { scalar $args{ldap_entry}->get_value($_) }
+                         @attributes;
+
+        if (($obj->FirstCustomFieldValue($cf_name) || '') eq ($value || '')) {
+            $self->_debug($obj->Name . ": Value '$value' is already set for '$cf_name'");
+            next;
+        }
+
+        $self->_debug($obj->Name . ": Adding object value '$value' for '$cf_name'");
+        next unless $args{import};
+
+        my ($ok, $msg) = $obj->AddCustomFieldValue( Field => $cf_name, Value => $value );
+        $self->_error($obj->Name . ": Couldn't add value '$value' for '$cf_name': $msg")
+            unless $ok;
+    }
+}
+
 =head2 import_groups import => 1|0
 
-Takes the results of the search from run_group_search
-and maps attributes from LDAP into RT::Group attributes
-using $RT::LDAPGroupMapping.
+Takes the results of the search from C<run_group_search>
+and maps attributes from LDAP into C<RT::Group> attributes
+using C<$RT::LDAPGroupMapping>.
 
-Creates groups if they don't exist
+Creates groups if they don't exist.
 
-Removes users from groups if they have been removed from the group on LDAP
+Removes users from groups if they have been removed from the group on LDAP.
 
 With no arguments, only prints debugging information.
-Pass import => 1 to actually change data.
+Pass C<--import> to actually change data.
 
 =cut
 
@@ -673,7 +940,7 @@ sub import_groups {
 
 =head3 run_group_search
 
-Set up the approviate arguments for a listing of users
+Set up the appropriate arguments for a listing of users.
 
 =cut
 
@@ -694,7 +961,7 @@ sub run_group_search {
 
 =head2 _import_group
 
-The user has run us with --import, so bring data in
+The user has run us with C<--import>, so bring data in.
 
 =cut
 
@@ -708,17 +975,18 @@ sub _import_group {
     my ($group_obj, $created) = $self->create_rt_group( %args, group => $group );
     return if $args{import} and not $group_obj;
     $self->add_group_members( %args, name => $group->{Name}, group => $group_obj, ldap_entry => $ldap_entry, new => $created );
+    # XXX TODO: support OCFVs for groups too
     return;
 }
 
 =head2 create_rt_group
 
-Takes a hashref of args to pass to RT::Group::Create
+Takes a hashref of args to pass to C<RT::Group::Create>
 Will try loading the group and will only create a new
-group if it can't find an existing group with the Name
-or EmailAddress arg passed in.
+group if it can't find an existing group with the C<Name>
+or C<EmailAddress> arg passed in.
 
-If $LDAPUpdateOnly is true, we will not create new groups
+If C<$LDAPUpdateOnly> is true, we will not create new groups
 but we will update existing ones.
 
 There is currently no way to prevent Group data from being
@@ -774,7 +1042,7 @@ sub create_rt_group {
 
 =head3 add_group_members
 
-Iterate over the list of values in the Member_Attr LDAP entry.
+Iterate over the list of values in the C<Member_Attr> LDAP entry.
 Look up the appropriate username from LDAP.
 Add those users to the group.
 Remove members of the RT Group who are no longer members
@@ -814,10 +1082,14 @@ sub add_group_members {
         if (exists $users->{lc $member}) {
             next unless $username = $users->{lc $member};
         } else {
+            my $attr    = lc($RT::LDAPGroupMapping->{Member_Attr_Value} || 'dn');
+            my $base    = $attr eq 'dn' ? $member : $RT::LDAPBase;
+            my $filter  = $attr eq 'dn'
+                            ? $RT::LDAPFilter
+                            : "(&$RT::LDAPFilter($attr=" . escape_filter_value($member) . "))";
             my @results = $self->_run_search(
-                base   => $RT::LDAPBase,
-                filter => "(&$RT::LDAPFilter($RT::LDAPGroupMapping->{Member_Attr_Value}="
-                            . escape_filter_value($member) . "))"
+                base   => $base,
+                filter => $filter,
             );
             unless ( @results ) {
                 $users->{lc $member} = undef;
@@ -871,7 +1143,7 @@ sub _get_group_members_from_ldap {
 =head2 _show_group
 
 Show debugging information about the group record we're going to import
-when the groups reruns us with --import
+when the groups reruns us with C<--import>.
 
 =cut
 
@@ -917,9 +1189,9 @@ sub _show_group_info {
 
 =head3 disconnect_ldap
 
-Disconnects from the LDAP server
+Disconnects from the LDAP server.
 
-Takes no arguments, returns nothing
+Takes no arguments, returns nothing.
 
 =cut
 
@@ -937,9 +1209,8 @@ sub disconnect_ldap {
 
 =head3 screendebug
 
-We always log to the RT log file with level debug 
-
-This duplicates the messages to the screen
+We always log to the RT log file with level 'debug'. This duplicates
+the messages to the screen.
 
 =cut
 
